#! /usr/bin/env ruby

require_relative 'textpatch'
require 'optionparser'
require 'shellwords'
require 'pathname'

FUSE = true

MOUNT_OVERLAY = FUSE ? %w(fuse-overlayfs) : %w(sudo mount -t overlay)
UMOUNT = FUSE ? %w(fusermount -u) : %w(sudo umount)

def cmd *args
  $stderr.puts "+ #{args.shelljoin}"
  system *args
  $?.success?
end

def umount dir
  cmd('mountpoint', '-q', dir) && (
    cmd(*UMOUNT, dir) || fail('Umount failed'))
end

Dir.chdir __dir__
base = 'ovrlay'

dirs = ['base']
tracks = Dir.enum_for(:glob, '*/*', base: File.join(base, 'mod/track')).to_h do |f|
  [File.basename(f), File.dirname(f)]
end
misc = Dir.glob('*', base: File.join(base, 'mod/misc')).to_set

tr_file = 'data/text/text.eng'
translate = NFS3Text.read File.read File.join base, 'base', tr_file

save = true
OptionParser.new do |opt|
  opt.on('--no-save') { save = false }
end.parse!

ARGV.each do |a|
  if tracks[a]
    dirs.unshift File.join('mod/track', tracks[a], a)
    translate[NFS3Text::TRACK_MAP[tracks[a]]] = a.split(/[ _]/).map(&:capitalize).join ' '
  elsif misc.member? a
    dirs.unshift File.join('mod/misc', a)
  else
    fail "Unknown mod #{a}"
  end
end

dirs.unshift 'save' if save
fail if dirs.size < 2

File.open File.join(base, 'save', tr_file), 'w' do |f|
  NFS3Text.write translate, f
end

dirs.map! {|d| File.join(base, d).gsub %r{([\\,:])}, '\\\1' }
work = File.join base, 'idiotism'
# note: a single mount option has an undocumented length limit, enough to reach
# by 3-4 mods... lowerdir+ works, but has weird syntax
ld = FUSE ? "lowerdir=#{dirs[1...].join ':'}," :
       dirs[1...].map {|x| "lowerdir+=#{x}," }.join
opts = "#{ld}upperdir=#{dirs[0]},workdir=#{work}"
tmp_mnt = File.join(base, 'mount')
real_mnt = 'mount'

umount real_mnt
umount tmp_mnt

cmd(*MOUNT_OVERLAY, '-o', opts, 'nothing', tmp_mnt) ||
  fail('Tmp mount failed')

cmd('mountpoint', '-q', real_mnt) ||
  cmd('ciopfs', tmp_mnt, real_mnt) ||
  fail('Real mout failed')
